
#include "DetectorConstruction.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "ScintillatorSD.hh"
#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"

#include "globals.hh"

#include "TVector3.h"

#include <sstream>
#include <algorithm>

DetectorConstruction::DetectorConstruction()
 : fLogicNeut(0), fScintillatorSD(0)
{
  //  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_lH2");
  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  //  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Pb");
  //  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_C");

  //  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

DetectorConstruction::~DetectorConstruction()
{;}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Neut LV
  //  G4ThreeVector neutSize(1*mm, 1*mm, 1*mm);
  G4ThreeVector neutSize(12*cm, 180*cm, 12*cm);
  //  G4ThreeVector neutSize(1*mm, 1*mm, 1*cm);
  //  G4ThreeVector neutSize(3*mm, 3*mm, 3*mm);
  G4ThreeVector neutPos(0, 0, neutSize.z());
  G4Box* solidNeut = new G4Box("Neut",
			       neutSize.x()/2,
			       neutSize.y()/2,
			       neutSize.z()/2);
  fLogicNeut = new G4LogicalVolume(solidNeut,fNeutMaterial,"Neut");

  // World LV
  G4ThreeVector worldSize(1.2*2*(std::abs(neutPos.x())+neutSize.x()/2),
			  1.2*2*(std::abs(neutPos.y())+neutSize.y()/2),
			  1.2*2*(std::abs(neutPos.z())+neutSize.z()/2));
  G4Box* solidWorld = new G4Box("World",
			    worldSize.x()/2,
			    worldSize.y()/2,
			    worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // Neut PV
  G4String neutName("Neut");
  new G4PVPlacement(0,
		    neutPos,
		    fLogicNeut,
		    neutName,
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  // add SD
  G4SDManager* SDman = G4SDManager::GetSDMpointer();
  fScintillatorSD = new ScintillatorSD("/ScintillatorBulk");
  SDman->AddNewDetector(fScintillatorSD);
  fLogicNeut->SetSensitiveDetector(fScintillatorSD);

  // colors
  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* neutColour = new G4VisAttributes(G4Colour::Yellow());
  fLogicNeut->SetVisAttributes(neutColour);
  
  // Print status
  G4cout << "### Neut consist of " 
         << fNeutMaterial->GetName() 
         <<  "  ###" << G4endl;

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}
