
#include "RunActionBasic.hh"
#include "ActionCommonVariable.hh"

#include "G4Run.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterGlobal.hh"
#include "TDataInitializerInterface.hh"

#include "TFile.h"
#include "TTree.h"

#include <fstream>
#include <iomanip>
#include <sstream>

RunActionBasic::RunActionBasic(TDataInitializerInterface* dataInitializerInterface)
  : fDataInitializerInterface(dataInitializerInterface), fFileOut(0) 
{
  fActionCommonVariable = ActionCommonVariable::GetInstance();
}

RunActionBasic::~RunActionBasic()
{
  delete fDataInitializerInterface;
}

void RunActionBasic::BeginOfRunAction(const G4Run* aRun)
{
  G4int id = aRun->GetRunID();
  G4cout << "### Run " << id << " start" << G4endl;

  // accumulate data by ROOT
  std::ostringstream oss;
  oss << "root/" << fActionCommonVariable->GetRunName() << std::setw(4) << std::setfill('0') << id << ".root";

  // over write warning
  std::ifstream ifileForCheck(oss.str().c_str());
  if(ifileForCheck){ // file exists
    if(fActionCommonVariable->GetOverWrite()=="ask"){
      G4cout << oss.str() << " already exists. over write[y], exit[n]>" << G4endl;    
      std::string yn;
      G4cin >> yn;
      if(!(yn == "y" || yn == "yes")){
	std::abort();
	//	std::exit(1);
      }
    }else if(fActionCommonVariable->GetOverWrite()=="y"){
      G4cout << oss.str() << " already exists. over write." << G4endl;    
    }else{
      G4cout << oss.str() << " already exists. exit." << G4endl;    
      std::abort();
      //      std::exit(1);      
    }
  }
  fFileOut = TFile::Open(oss.str().c_str(), "recreate");
  if(!fFileOut || fFileOut->IsZombie()) G4Exception(0, 0, FatalException, 0);

  fDataInitializerInterface->Initialize();

  if(fActionCommonVariable->GetUseTree()){
    // create event data branch. tree will be automatically added to current directory
    TTree* tree = new TTree(fActionCommonVariable->GetTreeName(),
			    fActionCommonVariable->GetTreeTitle());
    fDataInitializerInterface->CreateBranch(tree);

    // save tree name
    if(!gSimulationParameterGlobal) gSimulationParameterGlobal = new TSimulationParameterGlobal;
    gSimulationParameterGlobal->fTreeName = fActionCommonVariable->GetTreeName();
  }

  fDataInitializerInterface->AddParameters(fFileOut);
  fDataInitializerInterface->PrintParameters(fFileOut);

  fFileOut->Write();
}

void RunActionBasic::EndOfRunAction(const G4Run* aRun)
{
  G4cout << "\n Event:" << aRun->GetNumberOfEvent() << std::endl;

  fFileOut->Write();

  if(fActionCommonVariable->GetUseTree()){
    TTree* tree;
    gDirectory->GetObject(fActionCommonVariable->GetTreeName(), tree);
    tree->ResetBranchAddresses();
  }

  G4cout << G4endl;
}
