
#include "DipoleConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

DipoleConstruction::DipoleConstruction()
  : fLogicDipole(0), fLogicWorld(0)
{
  //  fDipoleMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_STAINLESS-STEEL");
  fDipoleMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Fe");
  fDipoleMotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

DipoleConstruction::~DipoleConstruction()
{;}

G4VPhysicalVolume* DipoleConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct Dipole LV
  fLogicDipole = ConstructSub();

  // dipole position
  G4ThreeVector dipolePos(0, 0, 6.8*m);

  // World LV
  G4ThreeVector worldSize(2*fDipoleSize.x(),
			  2*fDipoleSize.y(),
			  2*(fDipoleSize.z()+1.2*dipolePos.z()));
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // dipole PV
  new G4PVPlacement(0,
		    dipolePos,
		    fLogicDipole,
		    "Dipole",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicDipole->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* DipoleConstruction::ConstructSub()
{
  std::cout << "Createing Dipole" << std::endl;

  // dipole LV
  G4double dipolegap = 80*cm;
  G4ThreeVector dipoleSize1(7*m, 2*m, 3*m);
  G4ThreeVector dipolePos1(0, dipoleSize1.y()/2+dipolegap/2, 0);
  G4ThreeVector dipoleSize2(dipoleSize1.x()/2-dipolegap/2, dipolegap, dipoleSize1.z());
  G4ThreeVector dipolePos2(dipoleSize2.x()/2+dipolegap/2, 0, 0);
  G4ThreeVector dipoleMotherSize(dipoleSize2.x()+dipolePos2.x()*2,
				 dipoleSize1.y()+dipolePos1.y()*2,
				 dipoleSize1.z());
  fDipoleSize = dipoleMotherSize;

  G4Box* solidDipole1 = new G4Box("Dipole1",
				 dipoleSize1.x()/2,
				 dipoleSize1.y()/2,
				 dipoleSize1.z()/2);
  G4Box* solidDipole2 = new G4Box("Dipole2",
				 dipoleSize2.x()/2,
				 dipoleSize2.y()/2,
				 dipoleSize2.z()/2);
  G4LogicalVolume* logicDipole1 = new G4LogicalVolume(solidDipole1,fDipoleMaterial,"Dipole1");
  G4LogicalVolume* logicDipole2 = new G4LogicalVolume(solidDipole2,fDipoleMaterial,"Dipole2");

  G4Box* solidDipoleMother = new G4Box("DipoleMother",
				       dipoleMotherSize.x()/2,
				       dipoleMotherSize.y()/2,
				       dipoleMotherSize.z()/2);
  G4LogicalVolume* logicDipoleMother = new G4LogicalVolume(solidDipoleMother,fDipoleMotherMaterial,"DipoleMother");

  new G4PVPlacement(0,
		    dipolePos1,
		    logicDipole1,
		    "Dipole",
		    logicDipoleMother,
		    false,
		    0);
  dipolePos1.setY(-dipolePos1.y());
  new G4PVPlacement(0,
		    dipolePos1,
		    logicDipole1,
		    "Dipole",
		    logicDipoleMother,
		    false,
		    1);
  new G4PVPlacement(0,
		    dipolePos2,
		    logicDipole2,
		    "Dipole",
		    logicDipoleMother,
		    false,
		    2);
  dipolePos2.setX(-dipolePos2.x());
  new G4PVPlacement(0,
		    dipolePos2,
		    logicDipole2,
		    "Dipole",
		    logicDipoleMother,
		    false,
		    3);

  //  logicDipole1->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));
  logicDipole1->SetVisAttributes(new G4VisAttributes(G4Colour::Blue()));
  logicDipole2->SetVisAttributes(new G4VisAttributes(G4Colour::Blue()));

  return logicDipoleMother;
}
