
#include "TDataInitializerNeutronDetector.hh"

#include "TFile.h"
#include "TTree.h"

#include <iostream>

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"

ClassImp(TDataInitializerNeutronDetector)

TDataInitializerNeutronDetector::TDataInitializerNeutronDetector()
  : TDataInitializerInterface()
{;}

TDataInitializerNeutronDetector::~TDataInitializerNeutronDetector()
{;}

int TDataInitializerNeutronDetector::Initialize()
{
  delete gSimulationDataBeamArray; gSimulationDataBeamArray = 0;
  delete gSimulationDataScintillatorArray; gSimulationDataScintillatorArray = 0;
  gSimulationDataBeamArray = new TSimulationDataBeamArray;
  gSimulationDataScintillatorArray = new TSimulationDataScintillatorArray;

  return 1;
}

int TDataInitializerNeutronDetector::CreateBranch(TTree* tree)
{
  tree->Branch("beam", gSimulationDataBeamArray);
  tree->Branch("neutronDetector", gSimulationDataScintillatorArray);

  return 1;
}

int TDataInitializerNeutronDetector::AddParameters(TFile* file)
{
  file->Add(gSimulationParameterGlobal);
  file->Add(gSimulationParameterNeutronDetector);

  return 1;
}

int TDataInitializerNeutronDetector::PrintParameters(TFile* file)
{
  std::cout << "\n parameters of TDataInitializerNeutronDetector:" << std::endl;

  TSimulationParameterGlobal* simParaGlobal;
  TSimulationParameterNeutronDetector* simParaNeut;

  file->GetObject("parameterGlobal", simParaGlobal);
  file->GetObject("parameterNeutronDetector", simParaNeut);
  if(!simParaGlobal){
    std::cerr << " SimParaGlobal does not exist" << std::endl;
    return 0;
  }else if(!simParaNeut){
    std::cerr << " SimParaNeut does not exist" << std::endl;
    return 0;
  }
  std::cout << " " << *simParaGlobal << "\n"
	    << " " << *simParaNeut << "\n"
    	    << std::endl;

  return 1;
}

int TDataInitializerNeutronDetector::ClearBuffer()
{
  gSimulationDataScintillatorArray->clear();

  return 1;
}
